==> demokritos-0.2.0.tgz: demokritos-0.2.0/test/test_collection_paging_client.py <==
#!/usr/bin/env python

import httplib, socket, urllib

import sys
sys.path.append("../lib")
from app import parse_introspection
from atom import parse_atom


SERVER = "localhost:8000"

def urlsplit(url):
    return urllib.splithost(urllib.splittype(url)[1])


def request(server, method, path, body=None, headers=None):
    conn = httplib.HTTPConnection(server)
    try:
        print "trying %s %s to %s" % (method, path, server)
        if body is not None and headers is not None:
            conn.request(method, path, body, headers)
        else:
            conn.request(method, path)
        resp = conn.getresponse()
        conn.close()
        print "got %s %s" % (resp.status, resp.reason)
        return resp
    except socket.error:
        print "got socket error"
        return None

## Retrieving an Introspection Document (APP-07 5.1)
    
print "getting introspection document..."

collection_href = None
    
response = request(SERVER, "GET", "/introspection")
if response:
    assert response.status == 200
    doc = response.read()
    service = parse_introspection(doc)
    print "service:"
    for workspace in service.workspaces:
        print "\tworkspace:", workspace.title
        for collection in workspace.collections:
            print "\t\tcollection:", collection.title
            print "\t\t\thref =", collection.href
            print "\t\t\tmember-type =", collection.member_type
    collection1_href = service.workspaces[0].collections[0].href
    collection2_href = service.workspaces[0].collections[1].href
    collection3_href = service.workspaces[0].collections[2].href

raw_input("press return")

## List First Collection

if collection1_href:
    print "getting first collection..."

    server, path = urlsplit(collection1_href)
     
    response = request(server, "GET", path)
    if response:
        assert response.status == 200
        doc = response.read()
        collection = parse_atom(doc)        
        print "collection:"
        for link in collection.links:
            print "\t\tlink %s: %s" % (link.rel, link.href)
        for entry in collection.entries:
            print "\tmember:", entry.title.value
            print "\t\tupdated:", entry.updated.value
            for link in entry.links:
                print "\t\tlink %s: %s" % (link.rel, link.href)
            print "\t\tcontent src = %s" % entry.content.src

        assert collection.links[0].rel == "end"
        assert collection.links[0].href == collection1_href, "got %s" % collection.links[0].href

raw_input("press return")

## List Second Collection

if collection2_href:
    print "getting second collection..."

    server, path = urlsplit(collection2_href)
     
    response = request(server, "GET", path)
    if response:
        assert response.status == 200
        doc = response.read()
        collection = parse_atom(doc)        
        print "collection:"
        for link in collection.links:
            print "\t\tlink %s: %s" % (link.rel, link.href)
        for entry in collection.entries:
            print "\tmember:", entry.title.value
            print "\t\tupdated:", entry.updated.value
            for link in entry.links:
                print "\t\tlink %s: %s" % (link.rel, link.href)
            print "\t\tcontent src = %s" % entry.content.src

        assert len(collection.links) == 1
        assert collection.links[0].rel == "end"
        assert collection.links[0].href == collection2_href, "got %s" % collection.links[0].href

raw_input("press return")


## Creating a Resource

location = None
        
if collection2_href:
    print "POSTing to collection..."
    
    server, path = urlsplit(collection2_href)
     
    body = "Hello world!"
    headers = {"Content-type": "text/plain", "Title": "My Title"}
    response = request(server, "POST", path, body, headers)
    if response:
        assert response.status == 201
        location = response.getheader("Location")
        print "got location:", location
        
raw_input("press return")


## List Second Collection Again

next_href = None

if collection2_href:
    print "getting second collection again..."

    server, path = urlsplit(collection2_href)
     
    response = request(server, "GET", path)
    if response:
        assert response.status == 200
        doc = response.read()
        collection = parse_atom(doc)        
        print "collection:"
        for link in collection.links:
            print "\t\tlink %s: %s" % (link.rel, link.href)
        for entry in collection.entries:
            print "\tmember:", entry.title.value
            print "\t\tupdated:", entry.updated.value
            for link in entry.links:
                print "\t\tlink %s: %s" % (link.rel, link.href)
            print "\t\tcontent src = %s" % entry.content.src

        assert len(collection.links) == 2
        assert collection.links[0].rel == "next"
        assert collection.links[1].rel == "end"
        assert collection.links[0].href == "%s?index=10" % collection2_href, "got %s" % collection.links[0].href
        assert collection.links[1].href == "%s?index=10" % collection2_href, "got %s" % collection.links[0].href

        next_href = collection.links[0].href

raw_input("press return")


## Get second page

if next_href:
    print "getting second page of second collection..."

    server, path = urlsplit(next_href)
     
    response = request(server, "GET", path)
    if response:
        assert response.status == 200
        doc = response.read()
        collection = parse_atom(doc)        
        print "collection:"
        for link in collection.links:
            print "\t\tlink %s: %s" % (link.rel, link.href)
        for entry in collection.entries:
            print "\tmember:", entry.title.value
            print "\t\tupdated:", entry.updated.value
            for link in entry.links:
                print "\t\tlink %s: %s" % (link.rel, link.href)
            print "\t\tcontent src = %s" % entry.content.src

        assert len(collection.links) == 3
        assert collection.links[0].rel == "start"
        assert collection.links[1].rel == "previous"
        assert collection.links[2].rel == "end"
        assert collection.links[0].href == collection2_href
        assert collection.links[1].href == collection2_href
        assert collection.links[2].href == "%s?index=10" % collection2_href

raw_input("press return")


## List third collection

next_href = None

if collection1_href:
    print "getting third collection..."

    server, path = urlsplit(collection3_href)
     
    response = request(server, "GET", path)
    if response:
        assert response.status == 200
        doc = response.read()
        collection = parse_atom(doc)        
        print "collection:"
        for link in collection.links:
            print "\t\tlink %s: %s" % (link.rel, link.href)
        for entry in collection.entries:
            print "\tmember:", entry.title.value
            print "\t\tupdated:", entry.updated.value
            for link in entry.links:
                print "\t\tlink %s: %s" % (link.rel, link.href)
            print "\t\tcontent src = %s" % entry.content.src

        next_href = collection.links[0].href

raw_input("press return")


## Get next page of third collection

if next_href:
    print "getting next page of third collection..."

    server, path = urlsplit(next_href)
     
    response = request(server, "GET", path)
    if response:
        assert response.status == 200
        doc = response.read()
        collection = parse_atom(doc)        
        print "collection:"
        for link in collection.links:
            print "\t\tlink %s: %s" % (link.rel, link.href)
        for entry in collection.entries:
            print "\tmember:", entry.title.value
            print "\t\tupdated:", entry.updated.value
            for link in entry.links:
                print "\t\tlink %s: %s" % (link.rel, link.href)
            print "\t\tcontent src = %s" % entry.content.src

raw_input("press return")

==> demokritos-0.3.0.tgz: demokritos-0.3.0/test/test_collection_paging_client.py <==
#!/usr/bin/env python

import httplib, socket, urllib

import sys
sys.path.append("../lib")
from app import parse_introspection
from atom import parse_atom


SERVER = "localhost:8000"

def urlsplit(url):
    return urllib.splithost(urllib.splittype(url)[1])


def request(server, method, path, body=None, headers=None):
    conn = httplib.HTTPConnection(server)
    try:
        print "trying %s %s to %s" % (method, path, server)
        if body is not None and headers is not None:
            conn.request(method, path, body, headers)
        else:
            conn.request(method, path)
        resp = conn.getresponse()
        conn.close()
        print "got %s %s" % (resp.status, resp.reason)
        return resp
    except socket.error:
        print "got socket error"
        return None

## Retrieving an Introspection Document (APP-07 5.1)
    
print "getting introspection document..."

collection_href = None
    
response = request(SERVER, "GET", "/introspection")
if response:
    assert response.status == 200
    doc = response.read()
    service = parse_introspection(doc)
    print "service:"
    for workspace in service.workspaces:
        print "\tworkspace:", workspace.title
        for collection in workspace.collections:
            print "\t\tcollection:", collection.title
            print "\t\t\thref =", collection.href
            print "\t\t\tmember-type =", collection.member_type
    collection1_href = service.workspaces[0].collections[0].href
    collection2_href = service.workspaces[0].collections[1].href
    collection3_href = service.workspaces[0].collections[2].href

raw_input("press return")

## List First Collection

if collection1_href:
    print "getting first collection..."

    server, path = urlsplit(collection1_href)
     
    response = request(server, "GET", path)
    if response:
        assert response.status == 200
        doc = response.read()
        collection = parse_atom(doc)        
        print "collection:"
        for link in collection.links:
            print "\t\tlink %s: %s" % (link.rel, link.href)
        for entry in collection.entries:
            print "\tmember:", entry.title.value
            print "\t\tupdated:", entry.updated.value
            for link in entry.links:
                print "\t\tlink %s: %s" % (link.rel, link.href)
            print "\t\tcontent src = %s" % entry.content.src

        assert collection.links[0].rel == "last"
        assert collection.links[0].href == collection1_href, "got %s" % collection.links[0].href

raw_input("press return")

## List Second Collection

if collection2_href:
    print "getting second collection..."

    server, path = urlsplit(collection2_href)
     
    response = request(server, "GET", path)
    if response:
        assert response.status == 200
        doc = response.read()
        collection = parse_atom(doc)        
        print "collection:"
        for link in collection.links:
            print "\t\tlink %s: %s" % (link.rel, link.href)
        for entry in collection.entries:
            print "\tmember:", entry.title.value
            print "\t\tupdated:", entry.updated.value
            for link in entry.links:
                print "\t\tlink %s: %s" % (link.rel, link.href)
            print "\t\tcontent src = %s" % entry.content.src

        assert len(collection.links) == 1
        assert collection.links[0].rel == "last"
        assert collection.links[0].href == collection2_href, "got %s" % collection.links[0].href

raw_input("press return")


## Creating a Resource

location = None
        
if collection2_href:
    print "POSTing to collection..."
    
    server, path = urlsplit(collection2_href)
     
    body = "Hello world!"
    headers = {"Content-type": "text/plain", "Title": "My Title"}
    response = request(server, "POST", path, body, headers)
    if response:
        assert response.status == 201
        location = response.getheader("Location")
        print "got location:", location
        
raw_input("press return")


## List Second Collection Again

next_href = None

if collection2_href:
    print "getting second collection again..."

    server, path = urlsplit(collection2_href)
     
    response = request(server, "GET", path)
    if response:
        assert response.status == 200
        doc = response.read()
        collection = parse_atom(doc)        
        print "collection:"
        for link in collection.links:
            print "\t\tlink %s: %s" % (link.rel, link.href)
        for entry in collection.entries:
            print "\tmember:", entry.title.value
            print "\t\tupdated:", entry.updated.value
            for link in entry.links:
                print "\t\tlink %s: %s" % (link.rel, link.href)
            print "\t\tcontent src = %s" % entry.content.src

        assert len(collection.links) == 2
        assert collection.links[0].rel == "next"
        assert collection.links[1].rel == "last"
        assert collection.links[0].href == "%s?index=10" % collection2_href, "got %s" % collection.links[0].href
        assert collection.links[1].href == "%s?index=10" % collection2_href, "got %s" % collection.links[0].href

        next_href = collection.links[0].href

raw_input("press return")


## Get second page

if next_href:
    print "getting second page of second collection..."

    server, path = urlsplit(next_href)
     
    response = request(server, "GET", path)
    if response:
        assert response.status == 200
        doc = response.read()
        collection = parse_atom(doc)        
        print "collection:"
        for link in collection.links:
            print "\t\tlink %s: %s" % (link.rel, link.href)
        for entry in collection.entries:
            print "\tmember:", entry.title.value
            print "\t\tupdated:", entry.updated.value
            for link in entry.links:
                print "\t\tlink %s: %s" % (link.rel, link.href)
            print "\t\tcontent src = %s" % entry.content.src

        assert len(collection.links) == 3
        assert collection.links[0].rel == "first"
        assert collection.links[1].rel == "previous"
        assert collection.links[2].rel == "last"
        assert collection.links[0].href == collection2_href
        assert collection.links[1].href == collection2_href
        assert collection.links[2].href == "%s?index=10" % collection2_href

raw_input("press return")


## List third collection

next_href = None

if collection1_href:
    print "getting third collection..."

    server, path = urlsplit(collection3_href)
     
    response = request(server, "GET", path)
    if response:
        assert response.status == 200
        doc = response.read()
        collection = parse_atom(doc)        
        print "collection:"
        for link in collection.links:
            print "\t\tlink %s: %s" % (link.rel, link.href)
        for entry in collection.entries:
            print "\tmember:", entry.title.value
            print "\t\tupdated:", entry.updated.value
            for link in entry.links:
                print "\t\tlink %s: %s" % (link.rel, link.href)
            print "\t\tcontent src = %s" % entry.content.src

        next_href = collection.links[0].href

raw_input("press return")


## Get next page of third collection

if next_href:
    print "getting next page of third collection..."

    server, path = urlsplit(next_href)
     
    response = request(server, "GET", path)
    if response:
        assert response.status == 200
        doc = response.read()
        collection = parse_atom(doc)        
        print "collection:"
        for link in collection.links:
            print "\t\tlink %s: %s" % (link.rel, link.href)
        for entry in collection.entries:
            print "\tmember:", entry.title.value
            print "\t\tupdated:", entry.updated.value
            for link in entry.links:
                print "\t\tlink %s: %s" % (link.rel, link.href)
            print "\t\tcontent src = %s" % entry.content.src

raw_input("press return")
